@@ -1,5 +1,5 @@
 /* copy.c -- core functions for copying files and directories
-   Copyright (C) 89, 90, 91, 1995-2005 Free Software Foundation.
+   Copyright (C) 89, 90, 91, 1995-2006 Free Software Foundation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -1095,14 +1095,6 @@ copy_internal (char const *src_name, char const *dst_name,
 
 	  if (x->move_mode)
 	    {
-	      /* In move_mode, DEST may not be an existing directory.  */
-	      if (S_ISDIR (dst_sb.st_mode))
-		{
-		  error (0, 0, _("cannot overwrite directory %s"),
-			 quote (dst_name));
-		  return false;
-		}
-
 	      /* Don't allow user to move a directory onto a non-directory.  */
 	      if (S_ISDIR (src_sb.st_mode) && !S_ISDIR (dst_sb.st_mode))
 		{
@@ -1165,8 +1157,10 @@ copy_internal (char const *src_name, char const *dst_name,
 	  else if (! S_ISDIR (dst_sb.st_mode)
 		   && (x->unlink_dest_before_opening
 		       || (x->preserve_links && 1 < dst_sb.st_nlink)
-		       || (x->dereference == DEREF_NEVER
-			   && S_ISLNK (src_sb.st_mode))))
+		       || (!x->move_mode
+			   && x->dereference == DEREF_NEVER
+			   && S_ISLNK (src_sb.st_mode))
+		       ))
 	    {
 	      if (unlink (dst_name) != 0 && errno != ENOENT)
 		{
@@ -1252,40 +1246,49 @@ copy_internal (char const *src_name, char const *dst_name,
 		     quote_n (0, top_level_src_name),
 		     quote_n (1, top_level_dst_name));
 	      *copy_into_self = true;
+	      goto un_backup;
+	    }
+	  else if (x->dereference == DEREF_ALWAYS)
+	    {
+	      /* This happens when e.g., encountering a directory for the
+		 second or subsequent time via symlinks when cp is invoked
+		 with -R and -L.  E.g.,
+		 rm -rf a b c d; mkdir a b c d; ln -s ../c a; ln -s ../c b;
+		 cp -RL a b d
+	      */
 	    }
 	  else
 	    {
 	      error (0, 0, _("will not create hard link %s to directory %s"),
 		     quote_n (0, dst_name), quote_n (1, earlier_file));
+	      goto un_backup;
+	    }
+	}
+      else
+	{
+	  bool link_failed = (link (earlier_file, dst_name) != 0);
+
+	  /* If the link failed because of an existing destination,
+	     remove that file and then call link again.  */
+	  if (link_failed && errno == EEXIST)
+	    {
+	      if (unlink (dst_name) != 0)
+		{
+		  error (0, errno, _("cannot remove %s"), quote (dst_name));
+		  goto un_backup;
+		}
+	      link_failed = (link (earlier_file, dst_name) != 0);
 	    }
 
-	  goto un_backup;
+	  if (link_failed)
+	    {
+	      error (0, errno, _("cannot create hard link %s to %s"),
+		     quote_n (0, dst_name), quote_n (1, earlier_file));
+	      goto un_backup;
+	    }
+
+	  return true;
 	}
-
-      {
-	bool link_failed = (link (earlier_file, dst_name) != 0);
-
-	/* If the link failed because of an existing destination,
-	   remove that file and then call link again.  */
-	if (link_failed && errno == EEXIST)
-	  {
-	    if (unlink (dst_name) != 0)
-	      {
-		error (0, errno, _("cannot remove %s"), quote (dst_name));
-		goto un_backup;
-	      }
-	    link_failed = (link (earlier_file, dst_name) != 0);
-	  }
-
-	if (link_failed)
-	  {
-	    error (0, errno, _("cannot create hard link %s to %s"),
-		   quote_n (0, dst_name), quote_n (1, earlier_file));
-	    goto un_backup;
-	  }
-
-	return true;
-      }
     }
 
   if (x->move_mode)
